<?php

/**
 * @copyright (c) 2017 Chatwee sp. z o.o
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 */

namespace chatwee\chatwee\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use phpbb\exception\http_exception;

class listener implements EventSubscriberInterface
{
    public function __construct(
    	\phpbb\config\config $config,
    	\phpbb\template\template $template,
    	\phpbb\auth\auth $auth,
    	\phpbb\db\driver\driver_interface $db,
    	\phpbb\user $user,
    	\chatwee\chatwee\sso\sso_configuration $sso_configuration,
    	\chatwee\chatwee\sso\sso_session $sso_session,
    	\chatwee\chatwee\sso\sso_user $sso_user,
    	\chatwee\chatwee\sso\sso_manager $sso_manager,
    	$chatwee_users_table,
		$php_ext
    )
    {
        $this->config = $config;
        $this->template = $template;
        $this->auth = $auth;
        $this->db = $db;
        $this->user = $user;
		$this->sso_configuration = $sso_configuration;
        $this->sso_session = $sso_session;
        $this->sso_user = $sso_user;
        $this->sso_manager = $sso_manager;
        $this->chatwee_users_table = $chatwee_users_table;
		$this->php_ext = $php_ext;
    }

    public static function getSubscribedEvents()
    {
        return array(
            'core.page_header' => 'page_header_event_callback',
            'core.login_box_redirect' => 'login_box_redirect_event_callback',
            'core.session_create_after' => 'session_create_after_event_callback',
            'core.session_kill_after' => 'session_kill_after_event_callback'
        );
    }

    public function login_box_redirect_event_callback()
    {
    	if($this->config['single_sign_on_enable'])
		{
    		$this->set_chatwee_sso_credentials();
    		$this->sso_login();
    	}
    }

    public function session_create_after_event_callback()
    {
		if($this->config['single_sign_on_enable'])
		{
    		$this->set_chatwee_sso_credentials();
    		$this->sso_logout();
    	}
    }

    public function session_kill_after_event_callback()
    {
		if($this->config['single_sign_on_enable'])
		{
    		$this->set_chatwee_sso_credentials();
    		$this->sso_logout();
    	}
    }

    public function page_header_event_callback()
    {
    	if($this->config["chatwee_code"])
    	{
			$this->template->assign_vars(array(
				'CHATWEE_CODE' => htmlspecialchars_decode($this->config["chatwee_code"]),
			));
		}

		if($this->config['single_sign_on_enable'])
		{
			$this->set_chatwee_sso_credentials();
			$this->synchronize_user_sessions();
		}
    }

    public function set_chatwee_sso_credentials()
    {
		$this->sso_configuration->set_chat_id($this->config['single_sign_on_chat_id']);
        $this->sso_configuration->set_client_key($this->config['single_sign_on_key_api']);
    }

    public function synchronize_user_sessions()
    {

    	$is_user_logged_in = $this->user->data['user_id'] != ANONYMOUS;

		if($is_user_logged_in && !$this->sso_session->is_session_set())
		{
			$this->sso_login();
		}
		if($is_user_logged_in && $this->sso_session->is_session_set())
		{
			try
			{
				$is_session_valid = $this->sso_user->validate_session(array(
					'sessionId' => $this->sso_session->get_session_id(),
				));

				if($is_session_valid === false)
				{
					$this->sso_login();
				}
			}
			catch(http_exception $exception)
			{
    			global $phpbb_log;
				$user_id = $this->user->data['user_id'];
				$error_message = "Error while reinitializing SSO session:" . $exception->getMessage();
				$phpbb_log->add('critical', $user_id, $user->ip, 'CHATWEE_EXTENSION_ERROR', false, array($error_message));
			}
		}
    }

    public function sso_logout()
    {
        try
        {
            $this->sso_manager->logout_user();
        }
        catch(http_exception $exception)
        {
			global $phpbb_log;
			$user_id = (isset($user->data['user_id'])) ? $user->data['user_id'] : ANONYMOUS;
			$error_message = "Error while executing sso_logout:" . $exception->getMessage();
			$phpbb_log->add('critical', $user_id, $user->ip, 'CHATWEE_EXTENSION_ERROR', false, array($error_message));
        }
    }

    public function sso_login()
    {
    	$user_id = $this->user->data['user_id'];

        $chatwee_user_id = $this->get_chatwee_user_id($user_id);

        if(!$chatwee_user_id)
        {
            $chatwee_user_id = $this->sso_register($this->user->data);
        }

        try
        {
            $this->sso_manager->edit_user(array(
                'userId' => $chatwee_user_id,
                'login'  => $this->user->data['username'],
                'avatar' => $this->get_user_avatar($user_id),
                'isAdmin' => $this->is_chatwee_admin(),
            ));
        }
        catch(http_exception $exception)
        {
			if(strpos($exception->getMessage(), "USER_DOES_NOT_EXIST") !== false)
			{
				$chatwee_user_id = $this->sso_register($this->user->data);
			}
			else
			{
				global $phpbb_log;
				$error_message = "Error while executing sso_login:" . $exception->getMessage();
				$phpbb_log->add('critical', $user_id, $user->ip, 'CHATWEE_EXTENSION_ERROR', false, array($error_message));
			}
        }

        try
        {
            $this->sso_manager->login_user(array(
                'userId' => $chatwee_user_id,
            ));
        }
        catch(http_exception $exception)
        {
			global $phpbb_log;
			$error_message = "Error while executing sso_login:" . $exception->getMessage();
			$phpbb_log->add('critical', $user_id, $user->ip, 'CHATWEE_EXTENSION_ERROR', false, array($error_message));
        }
    }

    public function sso_register($user_data)
    {
    	$user_id = $user_data['user_id'];

        try
        {
            $chatwee_user_id = $this->sso_manager->register_user(array(
                'login' => $user_data['username'],
                'avatar' => $this->get_user_avatar($user_id),
                'isAdmin' => $this->is_chatwee_admin(),
            ));

            $this->assign_chatwee_user_id($user_id, $chatwee_user_id);
        }
        catch(http_exception $exception)
        {
			global $phpbb_log;
			$error_message = "Error while executing sso_register: " . $exception->getMessage();
			$phpbb_log->add('critical', $user_id, $user->ip, 'CHATWEE_EXTENSION_ERROR', false, array($error_message));
        }

        return $chatwee_user_id;
    }

    public function get_chatwee_user_id($user_id)
    {
        $sql_array = array(
            'user_id' => $user_id,
        );
        $sql = 'SELECT chatwee_user_id FROM ' . $this->chatwee_users_table . ' WHERE ' . $this->db->sql_build_array('SELECT', $sql_array);

        $result = $this->db->sql_query($sql);
        $row = $this->db->sql_fetchrow($result);
        $this->db->sql_freeresult($result);
        $chatwee_id = $row['chatwee_user_id'] ? $row['chatwee_user_id'] : null;
        return $chatwee_id;
    }

    public function assign_chatwee_user_id($user_id, $chatwee_user_id)
    {
        $sql_array = array(
            'user_id'    => $user_id,
            'chatwee_user_id' => $chatwee_user_id,
        );

        $sql = 'DELETE FROM ' . $this->chatwee_users_table . ' WHERE user_id = ' . $user_id;
		$this->db->sql_query($sql);

        $sql = 'INSERT INTO ' . $this->chatwee_users_table . ' ' . $this->db->sql_build_array('INSERT', $sql_array);
        $this->db->sql_query($sql);
    }

    public function get_user_avatar($user_id)
    {
        $sql_array = array(
            'user_id' => $user_id,
        );
        $sql = 'SELECT user_avatar FROM ' . USERS_TABLE . ' WHERE ' . $this->db->sql_build_array('SELECT', $sql_array);

        $result = $this->db->sql_query($sql);
        $row = $this->db->sql_fetchrow($result);
        $this->db->sql_freeresult($result);
        $row['user_avatar'] ? $avatar = $row['user_avatar'] : $avatar = false;
        $avatar ? $avatar = generate_board_url() . '/download/file.' . $this->php_ext . '?avatar=' . $avatar : '';

        return $avatar;
    }

    public function is_chatwee_admin()
    {
        return $this->auth->acl_get('a_') || ($this->auth->acl_get('m_') && $this->config['chatwee_global_moderators_as_admin']);
    }
}
