<?php

/**
 * @package phpBB Extension - Chatwee
 * @copyright (c) 2017 Chatwee sp. z o.o
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 */

namespace chatwee\chatwee\sso;

use phpbb\exception\http_exception;

class http_client
{
	protected $sso_configuration;

	private $response;

	private $response_status;

	private $response_object;

	const API_URL = 'http://chatwee-api.com/v2/';

	const USER_AGENT_HEADER = 'ChatweeV2 PHP SDK 1.02 RAW';

	public function __construct(
		\chatwee\chatwee\sso\sso_configuration $sso_configuration
	)
	{
		$this->sso_configuration = $sso_configuration;
	}

    public function get($method, $parameters)
    {
		if(!$this->sso_configuration->is_configuration_set())
		{
			return false;
		}

		$parameters['chatId'] = $this->sso_configuration->get_chat_id();
		$parameters['clientKey'] = $this->sso_configuration->get_client_key();

		$serialized_parameters = $this->serialize_parameters($parameters);
		$url = self::API_URL . $method . '?' . $serialized_parameters;

		self::call('GET', $url);
    }

	private function call($method, $url)
	{
		$curl = curl_init();

		$custom_user_agent = $this->sso_configuration->get_custom_user_agent();
		$user_agent = $custom_user_agent ? $custom_user_agent : self::USER_AGENT_HEADER;

		curl_setopt($curl, CURLOPT_TIMEOUT, 60);
		curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HTTPHEADER, array (
			'Accept: application/xml',
			'Content-Type: application/xml',
			'User-Agent: ' . $user_agent,
		));

		$this->response = curl_exec($curl);
		$this->response_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		curl_close($curl);

		$this->response_object = $this->response ? json_decode($this->response) : null;

		if($this->response_status != 200)
		{
			$response_error = $this->response_object->errorCode ? $this->response_object->errorCode : "ChatweeV2 PHP SDK unknown error: " . $this->response_status;
			$response_error .= " (" . $url . ")";
			throw new http_exception($this->response_status, $response_error);
		}
    }

	private function serialize_parameters($parameters)
	{
		if(!is_array($parameters) || count($parameters) == 0)
		{
			return '';
		}

		$result = '';
		foreach($parameters as $key => $value)
		{
			$result .= ($key . '=' . ($value ? urlencode($value) : $value) . '&');
		}

		$result = substr_replace($result, '', -1);
		return $result;
	}

	public function get_response()
	{
		return $this->response;
	}

	public function get_response_object()
	{
		return $this->response_object;
	}

	public function get_response_status()
	{
		return $this->response_status;
	}
}
