<?php

/**
 * @package phpBB Extension - Chatwee
 * @copyright (c) 2017 Chatwee sp. z o.o
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 */

namespace chatwee\chatwee\sso;

class sso_manager {

	public function __construct(
		\chatwee\chatwee\sso\sso_user $sso_user,
		\chatwee\chatwee\sso\sso_session $sso_session,
		\chatwee\chatwee\sso\sso_utils $sso_utils
	)
	{
		$this->sso_user = $sso_user;
		$this->sso_session = $sso_session;
		$this->sso_utils = $sso_utils;
	}

	public function register_user($parameters)
	{
    	if(!isSet($parameters['login']))
    	{
    		return false;
    	}

		$user_id = $this->sso_user->register(array(
			'login' => $parameters['login'],
			'isAdmin' => isSet($parameters['isAdmin']) ? $parameters['isAdmin'] : false,
			'avatar' => isSet($parameters['avatar']) ? $parameters['avatar'] : '',
		));

		return $user_id;
	}

    public function login_user($parameters)
    {
		if(!isSet($parameters['userId']))
		{
			return false;
		}
    	if(self::is_logged())
    	{
    		self::logout_user();
    	}

		$session_id = $this->sso_user->login(array(
			'userId' => $parameters['userId'],
			'userIp' => $this->sso_utils->get_user_ip(),
		));

		$this->sso_session->set_session_id($session_id);

		return $session_id;
    }

	public function logout_user()
	{
		if(!self::is_logged())
		{
			return false;
		}
		$session_id = $this->sso_session->get_session_id();

		$this->sso_user->remove_session(array(
			'sessionId' => $session_id,
		));

		$this->sso_session->clear_session_id();
	}

    public function edit_user($parameters)
    {
    	if(!isSet($parameters['login']))
    	{
    		return false;
    	}
		if(!isSet($parameters['userId']))
		{
			return false;
		}

		$edit_parameters = array(
			'userId' => $parameters['userId'],
			'login' => $parameters['login'],
			'avatar' => isSet($parameters['avatar']) ? $parameters['avatar'] : '',
		);

		if(isSet($parameters['isAdmin']))
		{
			$edit_parameters['isAdmin'] = $parameters['isAdmin'];
		}

		$this->sso_user->edit($edit_parameters);
    }

    private function is_logged()
    {
    	return $this->sso_session->is_session_set();
    }
}
