<?php

/**
 * @package phpBB Extension - Chatwee
 * @copyright (c) 2017 Chatwee sp. z o.o
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 */

namespace chatwee\chatwee\sso;

class sso_session
{
	protected $sso_configuration;

	const SESSION_DURATION = 86400;

	public function __construct(
		\phpbb\user $user,
		\phpbb\request\request $request,
		\chatwee\chatwee\sso\sso_configuration $sso_configuration
	)
	{
		$this->request = $request;
		$this->user = $user;
		$this->sso_configuration = $sso_configuration;
	}

	private function get_cookie_key()
	{
		if(!$this->sso_configuration->is_configuration_set())
		{
			return false;
		}
		return 'chatwee-SID-' . $this->sso_configuration->get_chat_id();
	}

    public function get_session_id()
    {
    	$cookie_key = $this->get_cookie_key();

    	$cookie_value = $this->request->variable($cookie_key, '', false, \phpbb\request\request_interface::COOKIE);

    	$session_id = $cookie_value ? $cookie_value : null;

		return $session_id;
    }

    public function set_session_id($session_id)
    {
		$host_chunks = explode('.', $this->request->server("HTTP_HOST", ''));

		$host_chunks = array_slice($host_chunks, -2);

		$cookie_domain = '.' . implode('.', $host_chunks);

		setcookie(self::get_cookie_key(), $session_id, time() + self::SESSION_DURATION, '/', $cookie_domain);
    }

    public function clear_session_id()
    {
		$host_chunks = explode('.', $this->request->server("HTTP_HOST", ''));

		$host_chunks = array_slice($host_chunks, -2);

		$cookie_domain = '.' . implode('.', $host_chunks);

		setcookie(self::get_cookie_key(), '', time() - 1, '/', $cookie_domain);
    }

    public function is_session_set()
    {
		return $this->get_session_id();
    }
}
